@@ -22,10 +22,14 @@ mkdir tmp
 printf "testing pdlzip..."
 cd "${objdir}"/tmp
 
-cat "${testdir}"/test1 > in || framework_failure
+cat "${testdir}"/test.txt > in || framework_failure
 fail=0
 
-"${LZIP}" -cd "${testdir}"/test1.lz > copy || fail=1
+"${LZIP}" -t "${testdir}"/test.lz || fail=1
+"${LZIP}" -cd "${testdir}"/test.lz > copy || fail=1
+cmp in copy || fail=1
+"${LZIP}" -t "${testdir}"/test.lzma || fail=1
+"${LZIP}" -cd "${testdir}"/test.lzma > copy || fail=1
 cmp in copy || fail=1
 
 for i in s4Ki 0 1 2 3 4 5 6 7 8s16 9s16 ; do
